from rest_framework import status
from rest_framework.exceptions import APIException

from ali_base_service.util.errors import get_error
from ali_base_service.util.handlers import get_error_response


class CustomValidationError(APIException):
    status_code = status.HTTP_200_OK

    def __init__(self, error_code, args=None):
        self.detail = get_error_response(get_error(error_code, args)).data



from django.utils.translation import ugettext as _

error_list = {
    101: {
        "sub_code": 101,
        "message": _("Does not exist"),
        "args": None
    },
    111: {
        "sub_code": 111,
        "message": _("Session is invalid/expired."),
        "args": None
    },
    116: {
        "sub_code": 116,
        "message": _("Following parameters have invalid values."),
        "args": None
    },
    117: {
        "sub_code": 117,
        "message": _('Following parameters are missing.'),
        "args": None
    },
    118: {
        "sub_code": 118,
        "message": _('Bad JSON'),
        "args": None
    },
    120: {
        "sub_code": 120,
        "message": _('Invalid basic header.'),
        "args": None
    },
    123: {
        "sub_code": 123,
        "message": _('User already exist.'),
        "args": None
    }
}


def get_error(error_code, args=None):
    error = error_list[error_code]

    if args is not None:
        error["args"] = args

    return error


def get_error_message(error_code):
    return error_list[error_code]["message"]




import json

from rest_framework import status
from rest_framework.response import Response


def convert_body_to_dict(body):
    decode = body.decode('utf-8')

    if decode == "":
        return {}

    return json.loads(decode)


def get_successful_response(data):
    return Response({
        "status": status.HTTP_200_OK,
        "latest_available_version": 1,
        "data": data,
        "error": None
    })


def get_error_response(error):
    return Response({
        "status": status.HTTP_400_BAD_REQUEST,
        "latest_available_version": 1,
        "data": None,
        "error": error
    })


def read_get_request_data(request_get):
    start = request_get.get("from")
    count = request_get.get("count")
    search_object = request_get.get("search_object", "{}")

    if start is not None:
        start = start.strip()

    if count is not None:
        count = count.strip()

    if search_object is not None:
        search_object = search_object.strip()

        if len(search_object) > 0:
            search_object = json.loads(search_object)
        else:
            search_object = None

    return search_object, start, count


def string_to_json(string):
    return json.loads(string)






import abc

from rest_framework.authentication import get_authorization_header
from rest_framework.views import APIView


from ali_base_service.util.handlers import *
from main import models


class BaseListAPI(APIView):
    __metaclass__  = abc.ABCMeta

    def get(self, request, **kwargs):
        self.request = request
        self.args=kwargs
        search_object, start, count = read_get_request_data(request.GET)
        order_by = request.GET.get('order_by')
        phone_number = request.META.get('HTTP_PHONE_NUMBER')

        return get_successful_response(self.get_query_set().list(
            serializer=self.get_serializer(),
            search_object=search_object,
            start=start,
            count=count,
            order_by=order_by,
            phone_number=phone_number
        ))

    @abc.abstractmethod
    def get_query_set(self):
        pass

    @abc.abstractmethod
    def get_serializer(self):
        pass

    def get_argument(self, key):
        return self.args[key]


class BaseGetAPI(APIView):
    __metaclass__ = abc.ABCMeta

    def get(self, request, id):
        phone_number = request.META.get('HTTP_PHONE_NUMBER')

        return get_successful_response(
            self.get_query_set().instance(
                instance_id=id,
                without_user_serializer=self.get_serializer(),
                phone_number=phone_number
            ))

    @abc.abstractmethod
    def get_query_set(self):
        pass

    @abc.abstractmethod
    def get_serializer(self):
        pass


def get_client(request):
    return None  # todo rename to get current user and return django user instead





import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django_ali_base_service',
    version='1.4.1',
    packages=find_packages(),
    install_requires=[
        'djangorestframework==3.5.3',
    ],
    include_package_data=True,
    license='BSD License',
    description='A base code for building webservices for django models based on Django Rest Framework',
    long_description=README,
    author='Alireza Ahmadi',
    author_email='alirezaahmadi69@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
